@@ -29,13 +29,19 @@
  * gst-launch utillity.
  */
 
-#include <string.h>
 
 #include "gst_private.h"
+#include <string.h>
 
 #include "gstparse.h"
 #include "gstinfo.h"
 
+#ifndef HAVE_MT_SAVE_FLEX
+/* the need for the mutex will go away with flex 2.5.6 */
+static gboolean flex_busy = FALSE;
+static GStaticRecMutex flex_lock = G_STATIC_REC_MUTEX_INIT;
+#endif
+
 extern GstElement *_gst_parse_launch (const gchar *, GError **);
 
 /**
@@ -137,17 +143,36 @@ GstElement *
 gst_parse_launch (const gchar * pipeline_description, GError ** error)
 {
   GstElement *element;
-  static GStaticMutex flex_lock = G_STATIC_MUTEX_INIT;
 
   g_return_val_if_fail (pipeline_description != NULL, NULL);
 
   GST_CAT_INFO (GST_CAT_PIPELINE, "parsing pipeline description %s",
       pipeline_description);
 
-  /* the need for the mutex will go away with flex 2.5.6 */
-  g_static_mutex_lock (&flex_lock);
+#ifndef HAVE_MT_SAVE_FLEX
+  g_static_rec_mutex_lock (&flex_lock);
+  if (flex_busy)
+    goto recursive_call;
+  flex_busy = TRUE;
+#endif
+
   element = _gst_parse_launch (pipeline_description, error);
-  g_static_mutex_unlock (&flex_lock);
+
+#ifndef HAVE_MT_SAVE_FLEX
+  flex_busy = FALSE;
+  g_static_rec_mutex_unlock (&flex_lock);
+#endif
 
   return element;
+
+  /* ERRORS */
+#ifndef HAVE_MT_SAVE_FLEX
+recursive_call:
+  {
+    GST_WARNING ("calls to gst_parse_launch() cannot be nested");
+    g_static_rec_mutex_unlock (&flex_lock);
+    g_warning ("calls to gst_parse_launch() cannot be nested");
+    return NULL;
+  }
+#endif
 }
