@@ -28,10 +28,10 @@
  * When defining own plugins, use the GST_BOILERPLATE ease gobject creation.
  */
 
+#include "gst_private.h"
 #include <stdio.h>
 #include <string.h>
 
-#include "gst_private.h"
 #include "gstghostpad.h"
 #include "gstutils.h"
 #include "gstinfo.h"
@@ -99,7 +99,7 @@ gst_util_set_value_from_string (GValue * value, const gchar * value_str)
 
   switch (G_VALUE_TYPE (value)) {
     case G_TYPE_STRING:
-      g_value_set_string (value, g_strdup (value_str));
+      g_value_set_string (value, value_str);
       break;
     case G_TYPE_ENUM:
     case G_TYPE_INT:{
@@ -482,6 +482,7 @@ overflow:
  * This function can potentially be very slow if denom > G_MAXUINT32.
  *
  * Returns: @val * @num / @denom, trying to avoid overflows.
+ * In the case of an overflow, this function returns G_MAXUINT64.
  */
 guint64
 gst_util_uint64_scale (guint64 val, guint64 num, guint64 denom)
@@ -528,7 +529,8 @@ do_int64:
  *
  * @num and @denom must be positive integers. @denom cannot be 0.
  *
- * Returns: @val * @num / @denom, avoiding overflow and loss of precision
+ * Returns: @val * @num / @denom, avoiding overflow and loss of precision.
+ * In the case of an overflow, this function returns G_MAXUINT64.
  */
 guint64
 gst_util_uint64_scale_int (guint64 val, gint num, gint denom)
@@ -985,7 +987,7 @@ gst_element_get_compatible_pad (GstElement * element, GstPad * pad,
  *
  * Returns: a string with the name of the state.
  */
-const gchar *
+G_CONST_RETURN gchar *
 gst_element_state_get_name (GstState state)
 {
   switch (state) {
@@ -1029,9 +1031,57 @@ gst_element_state_get_name (GstState state)
       return g_strdup_printf ("UNKNOWN!(%d)", state);
 #endif
   }
-  return "";
 }
 
+/**
+ * gst_element_state_change_return_get_name:
+ * @state_ret: a #GstStateChangeReturn to get the name of.
+ *
+ * Gets a string representing the given state change result.
+ *
+ * Returns: a string with the name of the state change result.
+ */
+G_CONST_RETURN gchar *
+gst_element_state_change_return_get_name (GstStateChangeReturn state_ret)
+{
+  switch (state_ret) {
+#ifdef GST_DEBUG_COLOR
+    case GST_STATE_CHANGE_FAILURE:
+      return "\033[01;31mFAILURE\033[00m";
+      break;
+    case GST_STATE_CHANGE_SUCCESS:
+      return "\033[01;32mSUCCESS\033[00m";
+      break;
+    case GST_STATE_CHANGE_ASYNC:
+      return "\033[01;33mASYNC\033[00m";
+      break;
+    case GST_STATE_CHANGE_NO_PREROLL:
+      return "\033[01;34mNO PREROLL\033[00m";
+      break;
+    default:
+      /* This is a memory leak */
+      return g_strdup_printf ("\033[01;35;41mUNKNOWN!\033[00m(%d)", state_ret);
+#else
+    case GST_STATE_CHANGE_FAILURE:
+      return "FAILURE";
+      break;
+    case GST_STATE_CHANGE_SUCCESS:
+      return "SUCCESS";
+      break;
+    case GST_STATE_CHANGE_ASYNC:
+      return "ASYNC";
+      break;
+    case GST_STATE_CHANGE_NO_PREROLL:
+      return "NO PREROLL";
+      break;
+    default:
+      /* This is a memory leak */
+      return g_strdup_printf ("UNKNOWN!(%d)", state_ret);
+#endif
+  }
+}
+
+
 /**
  * gst_element_factory_can_src_caps :
  * @factory: factory to query
@@ -1437,17 +1487,13 @@ gst_element_link_pads (GstElement * src, const gchar * srcpadname,
   if (srcpadname) {
     GST_CAT_DEBUG (GST_CAT_ELEMENT_PADS, "no link possible from %s:%s to %s",
         GST_DEBUG_PAD_NAME (srcpad), GST_ELEMENT_NAME (dest));
-    if (srcpad)
-      gst_object_unref (srcpad);
-    srcpad = NULL;
     if (destpad)
       gst_object_unref (destpad);
     destpad = NULL;
-  } else {
-    if (srcpad)
-      gst_object_unref (srcpad);
-    srcpad = NULL;
   }
+  if (srcpad)
+    gst_object_unref (srcpad);
+  srcpad = NULL;
 
   if (destpad) {
     /* loop through the existing pads in the destination */
@@ -1463,8 +1509,6 @@ gst_element_link_pads (GstElement * src, const gchar * srcpadname,
               GST_DEBUG_PAD_NAME (temp), GST_DEBUG_PAD_NAME (destpad));
           gst_object_unref (temp);
           gst_object_unref (destpad);
-          if (srcpad)
-            gst_object_unref (srcpad);
           return TRUE;
         }
         if (temp) {
@@ -1486,13 +1530,8 @@ gst_element_link_pads (GstElement * src, const gchar * srcpadname,
     GST_CAT_DEBUG (GST_CAT_ELEMENT_PADS, "no link possible from %s to %s:%s",
         GST_ELEMENT_NAME (src), GST_DEBUG_PAD_NAME (destpad));
     gst_object_unref (destpad);
-    if (srcpad)
-      gst_object_unref (srcpad);
     return FALSE;
   } else {
-    if (srcpad)
-      gst_object_unref (srcpad);
-    srcpad = NULL;
     if (destpad)
       gst_object_unref (destpad);
     destpad = NULL;
@@ -1602,8 +1641,9 @@ gst_element_link_pads_filtered (GstElement * src, const gchar * srcpadname,
       return TRUE;
     } else {
       GST_INFO ("Could not link elements");
+      gst_element_set_state (capsfilter, GST_STATE_NULL);
+      /* this will unlink and unref as appropriate */
       gst_bin_remove (GST_BIN (GST_OBJECT_PARENT (capsfilter)), capsfilter);
-      /* will unref and unlink as appropriate */
       return FALSE;
     }
   } else {
@@ -1621,6 +1661,9 @@ gst_element_link_pads_filtered (GstElement * src, const gchar * srcpadname,
  * existing pads that aren't linked yet. It will request new pads if necessary.
  * If multiple links are possible, only one is established.
  *
+ * Make sure you have added your elements to a bin or pipeline with
+ * gst_bin_add() before trying to link them.
+ *
  * Returns: TRUE if the elements could be linked, FALSE otherwise.
  */
 gboolean
@@ -1636,6 +1679,8 @@ gst_element_link (GstElement * src, GstElement * dest)
  * @...: the NULL-terminated list of elements to link in order.
  *
  * Chain together a series of elements. Uses gst_element_link().
+ * Make sure you have added your elements to a bin or pipeline with
+ * gst_bin_add() before trying to link them.
  *
  * Returns: TRUE on success, FALSE otherwise.
  */
@@ -1674,6 +1719,9 @@ gst_element_link_many (GstElement * element_1, GstElement * element_2, ...)
  * existing pads that aren't linked yet. It will request new pads if necessary.
  * If multiple links are possible, only one is established.
  *
+ * Make sure you have added your elements to a bin or pipeline with
+ * gst_bin_add() before trying to link them.
+ *
  * Returns: TRUE if the pads could be linked, FALSE otherwise.
  */
 gboolean
@@ -1815,6 +1863,7 @@ gst_element_unlink (GstElement * src, GstElement * dest)
         break;
     }
   }
+  gst_iterator_free (pads);
 }
 
 /**
@@ -1920,6 +1969,47 @@ gst_element_query_convert (GstElement * element, GstFormat src_format,
   return ret;
 }
 
+/**
+ * gst_element_seek_simple
+ * @element: a #GstElement to seek on
+ * @format: a #GstFormat to execute the seek in, such as #GST_FORMAT_TIME
+ * @seek_flags: seek options
+ * @seek_pos: position to seek to (relative to the start); if you are doing
+ *            a seek in #GST_FORMAT_TIME this value is in nanoseconds -
+ *            multiply with #GST_SECOND to convert seconds to nanoseconds or
+ *            with #GST_MSECOND to convert milliseconds to nanoseconds.
+ *
+ * Simple API to perform a seek on the given element, meaning it just seeks
+ * to the given position relative to the start of the stream. For more complex
+ * operations like segment seeks (e.g. for looping) or changing the playback
+ * rate or seeking relative to the last configured playback segment you should
+ * use gst_element_seek().
+ *
+ * In a completely prerolled PAUSED or PLAYING pipeline, seeking is always
+ * guaranteed to return %TRUE on a seekable media type or %FALSE when the media
+ * type is certainly not seekable (such as a live stream).
+ *
+ * Some elements allow for seeking in the READY state, in this
+ * case they will store the seek event and execute it when they are put to
+ * PAUSED. If the element supports seek in READY, it will always return %TRUE when
+ * it receives the event in the READY state.
+ *
+ * Returns: %TRUE if the seek operation succeeded (the seek might not always be
+ * executed instantly though)
+ *
+ * Since: 0.10.7
+ */
+gboolean
+gst_element_seek_simple (GstElement * element, GstFormat format,
+    GstSeekFlags seek_flags, gint64 seek_pos)
+{
+  g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
+  g_return_val_if_fail (seek_pos >= 0, FALSE);
+
+  return gst_element_seek (element, 1.0, format, seek_flags,
+      GST_SEEK_TYPE_SET, seek_pos, GST_SEEK_TYPE_NONE, 0);
+}
+
 /**
  * gst_pad_can_link:
  * @srcpad: the source #GstPad to link.
@@ -2015,6 +2105,7 @@ gst_pad_get_fixed_caps_func (GstPad * pad)
 
   g_return_val_if_fail (GST_IS_PAD (pad), NULL);
 
+  GST_OBJECT_LOCK (pad);
   if (GST_PAD_CAPS (pad)) {
     result = GST_PAD_CAPS (pad);
 
@@ -2039,6 +2130,8 @@ gst_pad_get_fixed_caps_func (GstPad * pad)
   result = gst_caps_new_empty ();
 
 done:
+  GST_OBJECT_UNLOCK (pad);
+
   return result;
 }
 
@@ -2325,7 +2418,7 @@ gst_buffer_join (GstBuffer * buf1, GstBuffer * buf2)
  * @dest: buffer to stamp
  * @src: buffer to stamp from
  *
- * Copies additional information (the timestamp, duration, and offset start 
+ * Copies additional information (the timestamp, duration, and offset start
  * and end) from one buffer to the other.
  *
  * This function does not copy any buffer flags or caps.
@@ -2390,18 +2483,40 @@ gst_pad_proxy_getcaps (GstPad * pad)
   if (element == NULL)
     return NULL;
 
-  iter = gst_element_iterate_pads (element);
-
+  /* value to hold the return, by default it holds ANY, the ref is taken by
+   * the GValue. */
   g_value_init (&ret, G_TYPE_POINTER);
   g_value_set_pointer (&ret, gst_caps_new_any ());
 
-  res = gst_iterator_fold (iter, (GstIteratorFoldFunction) intersect_caps_func,
-      &ret, pad);
+  iter = gst_element_iterate_pads (element);
+  while (1) {
+    res =
+        gst_iterator_fold (iter, (GstIteratorFoldFunction) intersect_caps_func,
+        &ret, pad);
+    switch (res) {
+      case GST_ITERATOR_RESYNC:
+        /* unref any value stored */
+        if ((caps = g_value_get_pointer (&ret)))
+          gst_caps_unref (caps);
+        /* need to reset the result again to ANY */
+        g_value_set_pointer (&ret, gst_caps_new_any ());
+        gst_iterator_resync (iter);
+        break;
+      case GST_ITERATOR_DONE:
+        /* all pads iterated, return collected value */
+        goto done;
+      default:
+        /* iterator returned _ERROR or premature end with _OK,
+         * mark an error and exit */
+        if ((caps = g_value_get_pointer (&ret)))
+          gst_caps_unref (caps);
+        g_value_set_pointer (&ret, NULL);
+        goto error;
+    }
+  }
+done:
   gst_iterator_free (iter);
 
-  if (res != GST_ITERATOR_DONE)
-    goto pads_changed;
-
   gst_object_unref (element);
 
   caps = g_value_get_pointer (&ret);
@@ -2413,10 +2528,11 @@ gst_pad_proxy_getcaps (GstPad * pad)
   return intersected;
 
   /* ERRORS */
-pads_changed:
+error:
   {
-    g_warning ("Pad list changed during capsnego for element %s",
+    g_warning ("Pad list returned error on element %s",
         GST_ELEMENT_NAME (element));
+    gst_iterator_free (iter);
     gst_object_unref (element);
     return NULL;
   }
@@ -2531,6 +2647,37 @@ gst_pad_query_position (GstPad * pad, GstFormat * format, gint64 * cur)
   return ret;
 }
 
+/**
+ * gst_pad_query_peer_position:
+ * @pad: a #GstPad on whose peer to invoke the position query on.
+ *       Must be a sink pad.
+ * @format: a pointer to the #GstFormat asked for.
+ *          On return contains the #GstFormat used.
+ * @cur: A location in which to store the current position, or NULL.
+ *
+ * Queries the peer of a given sink pad for the stream position.
+ *
+ * Returns: TRUE if the query could be performed.
+ */
+gboolean
+gst_pad_query_peer_position (GstPad * pad, GstFormat * format, gint64 * cur)
+{
+  gboolean ret = FALSE;
+  GstPad *peer;
+
+  g_return_val_if_fail (GST_IS_PAD (pad), FALSE);
+  g_return_val_if_fail (GST_PAD_IS_SINK (pad), FALSE);
+  g_return_val_if_fail (format != NULL, FALSE);
+
+  peer = gst_pad_get_peer (pad);
+  if (peer) {
+    ret = gst_pad_query_position (peer, format, cur);
+    gst_object_unref (peer);
+  }
+
+  return ret;
+}
+
 /**
  * gst_pad_query_duration:
  * @pad: a #GstPad to invoke the duration query on.
@@ -2562,6 +2709,38 @@ gst_pad_query_duration (GstPad * pad, GstFormat * format, gint64 * duration)
   return ret;
 }
 
+/**
+ * gst_pad_query_peer_duration:
+ * @pad: a #GstPad on whose peer pad to invoke the duration query on.
+ *       Must be a sink pad.
+ * @format: a pointer to the #GstFormat asked for.
+ *          On return contains the #GstFormat used.
+ * @duration: A location in which to store the total duration, or NULL.
+ *
+ * Queries the peer pad of a given sink pad for the total stream duration.
+ *
+ * Returns: TRUE if the query could be performed.
+ */
+gboolean
+gst_pad_query_peer_duration (GstPad * pad, GstFormat * format,
+    gint64 * duration)
+{
+  gboolean ret = FALSE;
+  GstPad *peer;
+
+  g_return_val_if_fail (GST_IS_PAD (pad), FALSE);
+  g_return_val_if_fail (GST_PAD_IS_SINK (pad), FALSE);
+  g_return_val_if_fail (format != NULL, FALSE);
+
+  peer = gst_pad_get_peer (pad);
+  if (peer) {
+    ret = gst_pad_query_duration (peer, format, duration);
+    gst_object_unref (peer);
+  }
+
+  return ret;
+}
+
 /**
  * gst_pad_query_convert:
  * @pad: a #GstPad to invoke the convert query on.
@@ -2602,6 +2781,43 @@ gst_pad_query_convert (GstPad * pad, GstFormat src_format, gint64 src_val,
   return ret;
 }
 
+/**
+ * gst_pad_query_peer_convert:
+ * @pad: a #GstPad, on whose peer pad to invoke the convert query on.
+ *       Must be a sink pad.
+ * @src_format: a #GstFormat to convert from.
+ * @src_val: a value to convert.
+ * @dest_format: a pointer to the #GstFormat to convert to.
+ * @dest_val: a pointer to the result.
+ *
+ * Queries the peer pad of a given sink pad to convert @src_val in @src_format
+ * to @dest_format.
+ *
+ * Returns: TRUE if the query could be performed.
+ */
+gboolean
+gst_pad_query_peer_convert (GstPad * pad, GstFormat src_format, gint64 src_val,
+    GstFormat * dest_format, gint64 * dest_val)
+{
+  gboolean ret = FALSE;
+  GstPad *peer;
+
+  g_return_val_if_fail (GST_IS_PAD (pad), FALSE);
+  g_return_val_if_fail (GST_PAD_IS_SINK (pad), FALSE);
+  g_return_val_if_fail (src_val >= 0, FALSE);
+  g_return_val_if_fail (dest_format != NULL, FALSE);
+  g_return_val_if_fail (dest_val != NULL, FALSE);
+
+  peer = gst_pad_get_peer (pad);
+  if (peer) {
+    ret = gst_pad_query_convert (peer, src_format, src_val, dest_format,
+        dest_val);
+    gst_object_unref (peer);
+  }
+
+  return ret;
+}
+
 /**
  * gst_atomic_int_set:
  * @atomic_int: pointer to an atomic integer
@@ -2653,19 +2869,10 @@ gulong
 gst_pad_add_data_probe (GstPad * pad, GCallback handler, gpointer data)
 {
   gulong sigid;
-  gboolean was_ghost = FALSE;
 
   g_return_val_if_fail (GST_IS_PAD (pad), 0);
   g_return_val_if_fail (handler != NULL, 0);
 
-  if (GST_IS_GHOST_PAD (pad)) {
-    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
-    if (!pad) {
-      return 0;
-    }
-    was_ghost = TRUE;
-  }
-
   GST_OBJECT_LOCK (pad);
   sigid = g_signal_connect (pad, "have-data", handler, data);
   GST_PAD_DO_EVENT_SIGNALS (pad)++;
@@ -2675,10 +2882,6 @@ gst_pad_add_data_probe (GstPad * pad, GCallback handler, gpointer data)
       GST_PAD_DO_BUFFER_SIGNALS (pad), GST_PAD_DO_EVENT_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
 
-  if (was_ghost) {
-    gst_object_unref (pad);
-  }
-
   return sigid;
 }
 
@@ -2697,19 +2900,10 @@ gulong
 gst_pad_add_event_probe (GstPad * pad, GCallback handler, gpointer data)
 {
   gulong sigid;
-  gboolean was_ghost = FALSE;
 
   g_return_val_if_fail (GST_IS_PAD (pad), 0);
   g_return_val_if_fail (handler != NULL, 0);
 
-  if (GST_IS_GHOST_PAD (pad)) {
-    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
-    if (!pad) {
-      return 0;
-    }
-    was_ghost = TRUE;
-  }
-
   GST_OBJECT_LOCK (pad);
   sigid = g_signal_connect (pad, "have-data::event", handler, data);
   GST_PAD_DO_EVENT_SIGNALS (pad)++;
@@ -2717,10 +2911,6 @@ gst_pad_add_event_probe (GstPad * pad, GCallback handler, gpointer data)
       GST_DEBUG_PAD_NAME (pad), GST_PAD_DO_EVENT_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
 
-  if (was_ghost) {
-    gst_object_unref (pad);
-  }
-
   return sigid;
 }
 
@@ -2739,19 +2929,10 @@ gulong
 gst_pad_add_buffer_probe (GstPad * pad, GCallback handler, gpointer data)
 {
   gulong sigid;
-  gboolean was_ghost = FALSE;
 
   g_return_val_if_fail (GST_IS_PAD (pad), 0);
   g_return_val_if_fail (handler != NULL, 0);
 
-  if (GST_IS_GHOST_PAD (pad)) {
-    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
-    if (!pad) {
-      return 0;
-    }
-    was_ghost = TRUE;
-  }
-
   GST_OBJECT_LOCK (pad);
   sigid = g_signal_connect (pad, "have-data::buffer", handler, data);
   GST_PAD_DO_BUFFER_SIGNALS (pad)++;
@@ -2759,10 +2940,6 @@ gst_pad_add_buffer_probe (GstPad * pad, GCallback handler, gpointer data)
       GST_DEBUG_PAD_NAME (pad), GST_PAD_DO_BUFFER_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
 
-  if (was_ghost) {
-    gst_object_unref (pad);
-  }
-
   return sigid;
 }
 
@@ -2776,19 +2953,9 @@ gst_pad_add_buffer_probe (GstPad * pad, GCallback handler, gpointer data)
 void
 gst_pad_remove_data_probe (GstPad * pad, guint handler_id)
 {
-  gboolean was_ghost = FALSE;
-
   g_return_if_fail (GST_IS_PAD (pad));
   g_return_if_fail (handler_id > 0);
 
-  if (GST_IS_GHOST_PAD (pad)) {
-    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
-    if (!pad) {
-      return;
-    }
-    was_ghost = TRUE;
-  }
-
   GST_OBJECT_LOCK (pad);
   g_signal_handler_disconnect (pad, handler_id);
   GST_PAD_DO_BUFFER_SIGNALS (pad)--;
@@ -2799,9 +2966,6 @@ gst_pad_remove_data_probe (GstPad * pad, guint handler_id)
       GST_PAD_DO_BUFFER_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
 
-  if (was_ghost) {
-    gst_object_unref (pad);
-  }
 }
 
 /**
@@ -2814,29 +2978,15 @@ gst_pad_remove_data_probe (GstPad * pad, guint handler_id)
 void
 gst_pad_remove_event_probe (GstPad * pad, guint handler_id)
 {
-  gboolean was_ghost = FALSE;
-
   g_return_if_fail (GST_IS_PAD (pad));
   g_return_if_fail (handler_id > 0);
 
-  if (GST_IS_GHOST_PAD (pad)) {
-    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
-    if (!pad) {
-      return;
-    }
-    was_ghost = TRUE;
-  }
-
   GST_OBJECT_LOCK (pad);
   g_signal_handler_disconnect (pad, handler_id);
   GST_PAD_DO_EVENT_SIGNALS (pad)--;
   GST_DEBUG ("removed event probe from pad %s:%s, now %d event probes",
       GST_DEBUG_PAD_NAME (pad), GST_PAD_DO_EVENT_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
-
-  if (was_ghost) {
-    gst_object_unref (pad);
-  }
 }
 
 /**
@@ -2849,19 +2999,9 @@ gst_pad_remove_event_probe (GstPad * pad, guint handler_id)
 void
 gst_pad_remove_buffer_probe (GstPad * pad, guint handler_id)
 {
-  gboolean was_ghost = FALSE;
-
   g_return_if_fail (GST_IS_PAD (pad));
   g_return_if_fail (handler_id > 0);
 
-  if (GST_IS_GHOST_PAD (pad)) {
-    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
-    if (!pad) {
-      return;
-    }
-    was_ghost = TRUE;
-  }
-
   GST_OBJECT_LOCK (pad);
   g_signal_handler_disconnect (pad, handler_id);
   GST_PAD_DO_BUFFER_SIGNALS (pad)--;
@@ -2869,9 +3009,6 @@ gst_pad_remove_buffer_probe (GstPad * pad, guint handler_id)
       GST_DEBUG_PAD_NAME (pad), GST_PAD_DO_BUFFER_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
 
-  if (was_ghost) {
-    gst_object_unref (pad);
-  }
 }
 
 /**
@@ -2903,6 +3040,8 @@ static void
 push_and_ref (GstPad * pad, GstEvent * event)
 {
   gst_pad_push_event (pad, gst_event_ref (event));
+  /* iterator refs pad, we unref when we are done with it */
+  gst_object_unref (pad);
 }
 
 /**
