@@ -33,7 +33,7 @@ G_BEGIN_DECLS
 #define GST_BASE_SINK_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_BASE_SINK,GstBaseSinkClass))
 #define GST_BASE_SINK_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_BASE_SINK, GstBaseSinkClass))
 #define GST_IS_BASE_SINK(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_BASE_SINK))
-#define GST_IS_BASE_SINK_CLASS(obj)	(G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_BASE_SINK))
+#define GST_IS_BASE_SINK_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_BASE_SINK))
 #define GST_BASE_SINK_CAST(obj)		((GstBaseSink *) (obj))
 
 /**
@@ -46,6 +46,7 @@ G_BEGIN_DECLS
 
 typedef struct _GstBaseSink GstBaseSink;
 typedef struct _GstBaseSinkClass GstBaseSinkClass;
+typedef struct _GstBaseSinkPrivate GstBaseSinkPrivate;
 
 /**
  * GstBaseSink:
@@ -95,12 +96,36 @@ struct _GstBaseSink {
       /* max amount of time a buffer can be late, -1 no limit. */
       gint64	     max_lateness;
     } ABI;
-    /* adding + 0 to mark ABI change to be undone later */
-    gpointer _gst_reserved[GST_PADDING_LARGE + 0];
+    gpointer _gst_reserved[GST_PADDING_LARGE - 1];
   } abidata;
 
+  GstBaseSinkPrivate *priv;
 };
 
+/**
+ * GstBaseSinkClass:
+ * @get_caps: Called to get sink pad caps from the subclass
+ * @set_caps: Notify subclass of changed caps
+ * @buffer_alloc: Subclasses can override to perform custom buffer allocations
+ * @get_times: Called to get the start and end times for synchronising
+ *             the passed buffer to the clock
+ * @start: Start processing. Ideal for opening resources in the subclass
+ * @stop: Stop processing. Subclasses should use this to close resources.
+ * @unlock: Unlock any pending access to the resource. Subclasses should
+ *          unblock any blocked function ASAP
+ * @event: Override this to handle events arriving on the sink pad
+ * @preroll: Called to present the preroll buffer if desired
+ * @render: Called when a buffer should be presented or output, at the
+ *          correct moment if the #GstBaseSink has been set to sync to
+ *          the clock.
+ * @async_play: Subclasses should override this when they need to perform
+ *              special processing when changing to the PLAYING state
+ *              asynchronously.  Called with the OBJECT_LOCK held.
+ *
+ * Subclasses can override any of the available virtual methods or not, as
+ * needed. At the minimum, the render method should be overridden to
+ * output/present buffers.
+ */
 struct _GstBaseSinkClass {
   GstElementClass parent_class;
 
@@ -130,18 +155,28 @@ struct _GstBaseSinkClass {
   GstFlowReturn (*preroll)      (GstBaseSink *sink, GstBuffer *buffer);
   GstFlowReturn (*render)       (GstBaseSink *sink, GstBuffer *buffer);
 
+  /* ABI additions */
+
+  /* when an ASYNC state change to PLAYING happens */ /* with LOCK */
+  GstStateChangeReturn (*async_play)   (GstBaseSink *sink);
+
   /*< private >*/
-  gpointer       _gst_reserved[GST_PADDING_LARGE];
+  gpointer       _gst_reserved[GST_PADDING_LARGE-1];
 };
 
 GType gst_base_sink_get_type(void);
 
+GstFlowReturn	gst_base_sink_wait_preroll 	(GstBaseSink *sink);
+
 void		gst_base_sink_set_sync 		(GstBaseSink *sink, gboolean sync);
 gboolean	gst_base_sink_get_sync 		(GstBaseSink *sink);
 
 void		gst_base_sink_set_max_lateness 	(GstBaseSink *sink, gint64 max_lateness);
 gint64		gst_base_sink_get_max_lateness 	(GstBaseSink *sink);
 
+void		gst_base_sink_set_qos_enabled 	(GstBaseSink *sink, gboolean enabled);
+gboolean	gst_base_sink_is_qos_enabled 	(GstBaseSink *sink);
+
 G_END_DECLS
 
 #endif /* __GST_BASE_SINK_H__ */
