@@ -192,25 +192,34 @@ gst_fd_src_update_fd (GstFdSrc * src)
 {
   struct stat stat_results;
 
-  src->fd = src->new_fd;
-  g_free (src->uri);
-  src->uri = g_strdup_printf ("fd://%d", src->fd);
+  if (src->fd != src->new_fd) {
+    GST_INFO_OBJECT (src, "Updating to fd %d", src->new_fd);
 
-  if (fstat (src->fd, &stat_results) < 0)
-    goto not_seekable;
+    src->fd = src->new_fd;
 
-  if (!S_ISREG (stat_results.st_mode))
-    goto not_seekable;
+    g_free (src->uri);
+    src->uri = g_strdup_printf ("fd://%d", src->fd);
 
-  /* Try a seek of 0 bytes offset to check for seekability */
-  if (lseek (src->fd, SEEK_CUR, 0) < 0)
-    goto not_seekable;
+    if (fstat (src->fd, &stat_results) < 0)
+      goto not_seekable;
 
-  src->seekable_fd = TRUE;
+    if (!S_ISREG (stat_results.st_mode))
+      goto not_seekable;
+
+    /* Try a seek of 0 bytes offset to check for seekability */
+    if (lseek (src->fd, 0, SEEK_CUR) < 0)
+      goto not_seekable;
+
+    GST_INFO_OBJECT (src, "marking fd %d as seekable", src->fd);
+    src->seekable_fd = TRUE;
+  }
   return;
 
 not_seekable:
-  src->seekable_fd = FALSE;
+  {
+    GST_INFO_OBJECT (src, "marking fd %d as NOT seekable", src->fd);
+    src->seekable_fd = FALSE;
+  }
 }
 
 static gboolean
@@ -278,7 +287,10 @@ gst_fd_src_set_property (GObject * object, guint prop_id, const GValue * value,
        * so it is reflected in get_properties and uri */
       GST_OBJECT_LOCK (object);
       if (GST_STATE (GST_ELEMENT (src)) <= GST_STATE_READY) {
+        GST_DEBUG_OBJECT (src, "state ready or lower, updating to use new fd");
         gst_fd_src_update_fd (src);
+      } else {
+        GST_DEBUG_OBJECT (src, "state above ready, not updating to new fd yet");
       }
       GST_OBJECT_UNLOCK (object);
       break;
@@ -309,7 +321,7 @@ gst_fd_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
 {
   GstFdSrc *src;
   GstBuffer *buf;
-  glong readbytes;
+  gssize readbytes;
   guint blocksize;
 
 #ifndef HAVE_WIN32
@@ -337,6 +349,8 @@ gst_fd_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
       gchar command;
       int res;
 
+      GST_LOG_OBJECT (src, "reading command");
+
       READ_COMMAND (src, command, res);
       if (res < 0) {
         GST_LOG_OBJECT (src, "no more commands");
@@ -355,6 +369,7 @@ gst_fd_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
 
   do {
     readbytes = read (src->fd, GST_BUFFER_DATA (buf), blocksize);
+    GST_LOG_OBJECT (src, "read %" G_GSSIZE_FORMAT, readbytes);
   } while (readbytes == -1 && errno == EINTR);  /* retry if interrupted */
 
   if (readbytes < 0)
@@ -368,7 +383,7 @@ gst_fd_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
   GST_BUFFER_TIMESTAMP (buf) = GST_CLOCK_TIME_NONE;
   src->curoffset += readbytes;
 
-  GST_DEBUG_OBJECT (psrc, "Read buffer of size %u.", readbytes);
+  GST_LOG_OBJECT (psrc, "Read buffer of size %" G_GSSIZE_FORMAT, readbytes);
 
   /* we're done, return the buffer */
   *outbuf = buf;
@@ -443,7 +458,7 @@ could_not_stat:
 
 /*** GSTURIHANDLER INTERFACE *************************************************/
 
-static guint
+static GstURIType
 gst_fd_src_uri_get_type (void)
 {
   return GST_URI_SRC;
